@@ -1,16 +1,25 @@
-#!/usr/bin/python2
-import gtk
-import os
-import sys
-import apt
-import commands
+#!/usr/bin/python3
 import gettext
-import tempfile
-from subprocess import Popen, PIPE
+import os
 import subprocess
-import vte
+import sys
+import locale
 
-gettext.install("pearlsources", "/usr/share/pearllinux/locale")
+import gi
+gi.require_version('Gtk', '3.0')
+gi.require_version('Vte', '2.91')
+from gi.repository import Gtk, Vte, GLib
+
+import apt
+import pearlcommon.aptdaemon
+
+# i18n
+APP = 'pearlsources'
+LOCALE_DIR = "/usr/share/pearllinux/locale"
+locale.bindtextdomain(APP, LOCALE_DIR)
+gettext.bindtextdomain(APP, LOCALE_DIR)
+gettext.textdomain(APP)
+_ = gettext.gettext
 
 (PKG_ID, PKG_CHECKED, PKG_NAME, PKG_INSTALLED_VERSION, PKG_REPO_VERSION, PKG_SORT_NAME) = range(6)
 
@@ -18,28 +27,27 @@ class Foreign_Browser():
 
     def __init__(self):
 
-        architecture = commands.getoutput("dpkg --print-architecture")
-
         self.downgrade_mode = (sys.argv[1] == "downgrade") # whether to downgrade or remove packages
 
-        glade_file = "/usr/lib/pearllinux/pearlSources/pearlSources.glade"
+        glade_file = "/usr/lib/pearllinux/pearlSources/pearlsources.glade"
 
-        self.builder = gtk.Builder()
+        self.builder = Gtk.Builder()
+        self.builder.set_translation_domain("pearlsources")
         self.builder.add_from_file(glade_file)
 
         self.window = self.builder.get_object("foreign_window")
         self.window.set_title(_("Foreign packages"))
-        self.window.set_icon_from_file("/usr/share/icons/hicolor/scalable/apps/software-sources.svg")
-        self.window.connect("destroy", gtk.main_quit)
-        self.builder.get_object("button_foreign_cancel").connect("clicked", gtk.main_quit)
+        self.window.set_icon_name("software-sources")
+        self.window.connect("destroy", Gtk.main_quit)
+        self.builder.get_object("button_foreign_cancel").connect("clicked", Gtk.main_quit)
         self.action_button = self.builder.get_object("button_foreign_action")
         self.action_button.connect("clicked", self.install)
         if self.downgrade_mode:
             self.action_button.set_label(_("Downgrade"))
-            self.builder.get_object("label_foreign_explanation").set_markup("<i>%s</i>" % _("The version of the following packages doesn't match the one from the repositories:"))
+            self.builder.get_object("label_foreign_explanation").set_markup("%s" % _("The version of the following packages doesn't match the one from the repositories:"))
         else:
             self.action_button.set_label(_("Remove"))
-            self.builder.get_object("label_foreign_explanation").set_markup("<i>%s</i>" % _("The packages below are installed on your computer but not present in the repositories:"))
+            self.builder.get_object("label_foreign_explanation").set_markup("%s" % _("The packages below are installed on your computer but not present in the repositories:"))
         self.action_button.set_sensitive(False)
 
         self.select_button = self.builder.get_object("button_foreign_select")
@@ -47,76 +55,60 @@ class Foreign_Browser():
         self.select_button.set_label(_("Select All"))
         self.select_button_selects_all = True
 
-        self.builder.get_object("button_foreign_cancel").set_label(_("Cancel"))
-        self.builder.get_object("label_foreign_warning").set_markup("<b>%s</b>" % _("WARNING"))
-        self.builder.get_object("label_foreign_review").set_markup("<i>%s</i>" % _("Review the information below very carefully before validating the command:"))
-
-        self.model = gtk.ListStore(str, bool, str, str, str, str)
+        self.model = Gtk.ListStore(str, bool, str, str, str, str)
         # PKG_ID, PKG_CHECKED, PKG_NAME, PKG_INSTALLED_VERSION, PKG_REPO_VERSION, PKG_SORT_NAME
 
         treeview = self.builder.get_object("treeview_foreign_pkgs")
         treeview.set_model(self.model)
-        self.model.set_sort_column_id(PKG_SORT_NAME, gtk.SORT_ASCENDING)
+        self.model.set_sort_column_id(PKG_SORT_NAME, Gtk.SortType.ASCENDING)
 
-        cr = gtk.CellRendererToggle()
+        cr = Gtk.CellRendererToggle()
         cr.connect("toggled", self.toggled)
-        col = gtk.TreeViewColumn("", cr)
+        col = Gtk.TreeViewColumn("", cr)
         col.set_cell_data_func(cr, self.datafunction_checkbox)
         treeview.append_column(col)
         col.set_sort_column_id(PKG_CHECKED)
 
-        r = gtk.CellRendererText()
-        col = gtk.TreeViewColumn(_("Package"), r, markup = PKG_NAME)
+        r = Gtk.CellRendererText()
+        col = Gtk.TreeViewColumn(_("Package"), r, markup = PKG_NAME)
         treeview.append_column(col)
         col.set_sort_column_id(PKG_NAME)
 
-        r = gtk.CellRendererText()
-        col = gtk.TreeViewColumn(_("Installed version"), r, markup = PKG_INSTALLED_VERSION)
+        r = Gtk.CellRendererText()
+        col = Gtk.TreeViewColumn(_("Installed version"), r, markup = PKG_INSTALLED_VERSION)
         treeview.append_column(col)
         col.set_sort_column_id(PKG_INSTALLED_VERSION)
 
         if self.downgrade_mode:
-            r = gtk.CellRendererText()
-            col = gtk.TreeViewColumn(_("Repository version"), r, markup = PKG_REPO_VERSION)
+            r = Gtk.CellRendererText()
+            col = Gtk.TreeViewColumn(_("Repository version"), r, markup = PKG_REPO_VERSION)
             treeview.append_column(col)
             col.set_sort_column_id(PKG_REPO_VERSION)
 
+        self.apt = pearlcommon.aptdaemon.APT(self.window)
+
         cache = apt.Cache()
+
+        # python-apt doesn't give us a constant for this value
+        # it's "required" in English, but it could be anything in
+        # other languages
+        required_priority = cache['dpkg'].installed.priority
+
         for key in cache.keys():
             pkg = cache[key]
             if (pkg.is_installed):
                 candidate_version = pkg.candidate.version
                 installed_version = pkg.installed.version
 
-                if not pkg.candidate.downloadable:
-                    # The candidate is not downloadable...
-                    # See if there's a version that is...
-                    best_version = None
-                    for version in pkg.versions:
-                        if version.downloadable:
-                            if best_version is None:
-                                best_version = version
-                            else:
-                                if version.policy_priority > best_version.policy_priority:
-                                    best_version = version
-                                elif version.policy_priority == best_version.policy_priority:
-                                    # same priorities, compare version
-                                    if commands.getoutput("dpkg --compare-versions %s gt %s && echo 'OK'" % (version.version, best_version.version)) == "OK":
-                                        best_version = version
-
-                    if self.downgrade_mode:
-                        if best_version is not None:
-                            for origin in best_version.origins:
-                                iter = self.model.insert_before(None, None)
-                                self.model.set_value(iter, PKG_ID, "%s=%s" % (pkg.name, best_version.version))
-                                self.model.set_value(iter, PKG_CHECKED, False)
-                                self.model.set_value(iter, PKG_NAME, "<b>%s</b>" % pkg.name)
-                                self.model.set_value(iter, PKG_INSTALLED_VERSION, installed_version)
-                                self.model.set_value(iter, PKG_REPO_VERSION, "%s (%s)" % (best_version.version, origin.archive))
-                                self.model.set_value(iter, PKG_SORT_NAME, "%s %s" % (best_version.source_name, pkg.name))
-                                break
-                    else:
-                        if best_version is None:
+                if not self.downgrade_mode:
+                    # remove mode
+                    # Find packages which aren't downloadable
+                    if not pkg.candidate.downloadable:
+                        downloadable = False
+                        for version in pkg.versions:
+                            if version.downloadable:
+                                downloadable = True
+                        if not downloadable and pkg.essential == False and pkg.installed.priority != required_priority:
                             iter = self.model.insert_before(None, None)
                             self.model.set_value(iter, PKG_ID, "%s" % (pkg.name))
                             self.model.set_value(iter, PKG_CHECKED, False)
@@ -124,20 +116,57 @@ class Foreign_Browser():
                             self.model.set_value(iter, PKG_INSTALLED_VERSION, installed_version)
                             self.model.set_value(iter, PKG_REPO_VERSION, "")
                             self.model.set_value(iter, PKG_SORT_NAME, "%s" % (pkg.name))
+                else:
+                    # downgrade mode
+                    # Find packages which candidate isn't available
+                    best_version = None
+                    if not pkg.candidate.downloadable:
+                        # foreign packages which installed version doesn't exist in the repositories
+                        for version in pkg.versions:
+                            if not version.downloadable:
+                                continue
+                            if version.version == installed_version:
+                                continue
+
+                            if best_version is None:
+                                best_version = version
+                            else:
+                                if version.policy_priority > best_version.policy_priority:
+                                    best_version = version
+                                elif version.policy_priority == best_version.policy_priority:
+                                    # same priorities, compare version
+                                    return_code = subprocess.call(["dpkg", "--compare-versions", version.version, "gt", best_version.version])
+                                    if return_code == 0:
+                                        best_version = version
+                    elif candidate_version == installed_version:
+                        # packages which installed version isn't the one available from the highest priority repo
+                        # typical example is an ubuntu version installed despite a inferior 700 version being available
+                        best_version = None
+                        for version in pkg.versions:
+                            if not version.downloadable:
+                                continue
+                            if version.version == installed_version:
+                                continue
+                            if version.policy_priority > 500:
+                                if best_version is None or version.policy_priority > best_version.policy_priority:
+                                    best_version = version
+
+                    if best_version is not None:
+                        for origin in best_version.origins:
+                            iter = self.model.insert_before(None, None)
+                            self.model.set_value(iter, PKG_ID, "%s=%s" % (pkg.name, best_version.version))
+                            self.model.set_value(iter, PKG_CHECKED, False)
+                            self.model.set_value(iter, PKG_NAME, "<b>%s</b>" % pkg.name)
+                            self.model.set_value(iter, PKG_INSTALLED_VERSION, installed_version)
+                            self.model.set_value(iter, PKG_REPO_VERSION, "%s (%s)" % (best_version.version, origin.archive))
+                            self.model.set_value(iter, PKG_SORT_NAME, "%s %s" % (best_version.source_name, pkg.name))
+                            break
 
         treeview.show()
         treeview.connect("row-activated", self.treeview_row_activated)
         self.window.show_all()
 
-        parent_window_xid = int(sys.argv[2])
-        try:
-            parent = gtk.gdk.window_foreign_new(parent_window_xid)
-            self.window.realize()
-            self.window.window.set_transient_for(parent)
-        except:
-            pass
-
-    def datafunction_checkbox(self, column, cell, model, iter):
+    def datafunction_checkbox(self, column, cell, model, iter, data):
         cell.set_property("activatable", True)
         checked = model.get_value(iter, PKG_CHECKED)
         if (checked):
@@ -167,26 +196,33 @@ class Foreign_Browser():
             self.action_button.set_sensitive(False)
 
     def install (self, button):
-        self.builder.get_object("notebook1").set_current_page(1)
-        term = vte.Terminal()
-        pid = term.fork_command('dash')
-        term.set_emulation('xterm')
-
         foreign_packages = []
         iter = self.model.get_iter_first()
         while (iter != None):
             if (self.model.get_value(iter, PKG_CHECKED)):
                 foreign_packages.append(self.model.get_value(iter, PKG_ID))
             iter = self.model.iter_next(iter)
-
         if self.downgrade_mode:
-            term.feed_child("apt-get install %s\n" % " ".join(foreign_packages))
+            self.builder.get_object("stack1").set_visible_child_name("vte")
+            terminal = Vte.Terminal()
+            terminal.spawn_sync(Vte.PtyFlags.DEFAULT, os.environ['HOME'], ["/bin/dash"], [], GLib.SpawnFlags.DO_NOT_REAP_CHILD, None, None,)
+            cmd = "apt-get install %s\n" % " ".join(foreign_packages)
+            # try/catch VTE call due to regressions and obscure documentation
+            try:
+                # Pearl 19.x
+                terminal.feed_child(cmd, -1)
+            except:
+                # LMDE 4
+                terminal.feed_child(cmd.encode("UTF-8"))
+            terminal.show()
+            self.builder.get_object("box_vte").add(terminal)
+            self.builder.get_object("box_vte").show_all()
         else:
-            term.feed_child("apt-get remove --purge %s\n" % " ".join(foreign_packages))
+            self.apt.set_finished_callback(self.exit)
+            self.apt.remove_packages(foreign_packages)
 
-        term.show()
-        self.builder.get_object("vbox_vte").add(term)
-        self.builder.get_object("vbox_vte").show_all()
+    def exit(self, transaction=None, exit_state=None):
+        sys.exit(0)
 
     def select_all (self, button):
         iter = self.model.get_iter_first()
@@ -203,4 +239,4 @@ class Foreign_Browser():
 
 if __name__ == "__main__":
     foreign_browser = Foreign_Browser()
-    gtk.main()
+    Gtk.main()
